function [xr] = bisec(a,rt,tol,lmt)
%binec.m bisection method
% 係数ベクトル a の代数方程式 f の解を挟みうち法（False Position Method）
% で求める．
% このときの解の精度は tol とする．
% 書式
% xr = bisec(a,rt,tol)
% 引数 a : 代数方程式 f の係数ベクトル
% f(x) = a(1)+a(2)*x+a(3)*x^2+ ... +a(n)*x^(n-1)
% rt: 解が存在する配列
% rt = [x1 x2] 解は x = x1 ～ x2 の間に存在すると仮定する
% tol: 計算精度 ( 実数 )
% lmt: ループ限界 ( 整数 )
% 指定した回数で解が得られなかった
% 戻り値 xr: 解
    x1 = rt(1); x2 = rt(2);
    % エラー定義
    ME1 = MException('BisctionMethodError:OutRange', ...
                     '区間[%f %f]に解は存在しません',rt(1),rt(2));
    ME2 = MException('BisctionMethodError:OutCount', ...
                     ' 指定回数 %d で解が求められませんでした ',lmt);
    rtf1 = polyval(a,x1); rtf2 = polyval(a,x2);
    xm = x1 - (x2 - x1)*rtf1/(rtf2 - rtf1);
    cnt = 1;
    while abs(rtf1-rtf2) > tol
        sgnf1 = sign(rtf1); % f(x1) の符号とり出し
        sgnf2 = sign(rtf2); % f(x2) の符号とり出し
        try % 区間内の解の有無のチェック
            if sgnf1 == sgnf2 % f(x1) と f(x2) が同符号ならば解なし
                xm = NaN;
                error(ME1.identifier); % エラーを発生
            end
        catch
            error(ME1.identifier,'%s\n%s',ME1.identifier,ME1.message);
        end
        xm = x1 - (x2 - x1)*rtf1/(rtf2 - rtf1); % False Position Method
        fm = polyval(a,xm); sgnfm = sign(fm);
        if sgnfm == sgnf1
            x1 = xm; rtf1 = polyval(a,x1);
        end
        if sgnfm == sgnf2
            x2 = xm; rtf2 = polyval(a,x2);
        end
        % 解の判定
        if abs(fm) <= tol
            break;
        end
        try % 無限ループ防止用
            cnt = cnt + 1;
            if cnt > lmt
                xm = NaN;
                error(ME2.identifier); % エラーを発生
            end
        catch
            error(ME2.identifier,'%s\n%s',ME2.identifier,ME2.message);
        end
    end
    xr = xm;
end
