function [ x,k ] = solvSORerr(A,b,inix,varargin)
%solvSOR.m SOR法(逐次緩和法：Successive Over-Relaxation Method)
% SOR 法による行列方程式 Ax = b の解法
% 書式：[x,k] = solvSOR(A,b,inix,STol,w)
% STol,w は省略可．ただし，STol を省略し w を指定する場合は STol に空行列
% を指定すること．
% 入力引数
% A：係数行列 / b：定数ベクトル
% inix：解の初期値
% STol：計算精度 省略時はデフォルト値(1e-5)
% w：スカラー 緩和係数 w=1でGauss-Seidel 法での計算
% 出力（戻り値）
% x：解ベクトル / k：スカラー 収束回数
% エラーコード
% x = [] エラー
% k = NaN 行列 A が正方行列ではない
% k = -inf 行列 A の対角要素に 0 が含まれている
    [M,N] = size(A);
    % パラメーターのエラーチェック
    if (M ~= N) % エラー：正方行列のチェック
        x = []; k = NaN;    return;
    end
    dA = diag(A);
    if ~all(dA) % エラー：対角要素のチェック
        x = []; k = -inf;   return;
    end
    switch nargin
    case 3  % STol,w 省略時
            STol = 1e-5; % デフォルトの収束精度
            % スペクトル半径の試算
            [SpcRmin,SpcRmax] = SpecResdu(A);
            if SpcRmin == SpcRmax
                rho = SpcRmax;
            else
                rho = (SpcRmax + SpcRmin)/2;
            end
            w = 2 / (1+sqrt(1-rho^2)); % 緩和係数の計算
    case 4 % w 省略時
            STol = varargin{1};
            % スペクトル半径の試算
            [SpcRmin,SpcRmax] = SpecResdu(A);
            if SpcRmin == SpcRmax
                rho = SpcRmax;
            else
                rho = (SpcRmax + SpcRmin)/2;
            end
            w = 2 / (1+sqrt(1-rho^2));
    case 5 % 引数省略なし
            STol = varargin{1,1};
            if isempty(STol)
                STol = 1e-5;
            end
            w = varargin{1,2};
            % 緩和係数の範囲チェック
            if(1>=w)||(w>=2) % 緩和係数の範囲外
                error(['指定した緩和係数 %f では発散する',...
                       '可能性があります\n',...
                       '緩和係数の値を検討してください'],w);
            end
    end
    n = length(b); x = inix; k = 0;
    dX = ones(n,1);
    while (norm(dX) / norm(x)) > STol
        for i = 1:n
            dX(i) = b(i) ./ dA(i);
            dX(i) = dX(i) - A(i,:)*x./dA(i);
            x(i) = x(i) + w*dX(i); % SOR
        end
        k = k + 1;
    end
end

function varargout = SpecResdu(A)
%SpecResdu.m 行列 A のスペクトル半径を試算する．
% スペクトル半径の計算
% min{|A(i,j)|/|A(i,i)|} <= ρ <= max{|A(i,j)|/|A(i,i)|}
% SOR 法による連立方程式の解法のための緩和係数を見積もるための
% スペクトル半径を試算する．
% 書式：
% [SpcRmin,SpcRmax,r] = SpecResdu(A)
% 引数： A 計算対象の行列
% 戻り値： SpecRmin 最小スペクトル半径(必修)
%         SpecRmax 最大スペクトル半径(必修)
%         r スペクトル半径ベクトル（オプション）
    [~,n] = size(A);
    r = zeros(1,n); d = diag(A);
    for i = 1:n
        for j = 1:n
            if i ~= j
                r(i) = abs(A(i,j)) / abs(d(i));
            end
        end
    end
    varargout = cell(nargout);
    varargout(1) = {min(r)}; % SpecRmin
    varargout(2) = {max(r)}; % SpecRmax
    if 3 == nargout
        varargout(3) = {r};
    end
end
